#include <cassert>

template<class OperatorType, class DiscFuncType, class BitVectorType>
inline
DiscFuncType BlockGSStep<OperatorType, DiscFuncType, BitVectorType>::getSol()
{
    return *(this->x_);
}

template<class OperatorType, class DiscFuncType, class BitVectorType>
inline 
void BlockGSStep<OperatorType, DiscFuncType, BitVectorType>::
residual(int index, VectorBlock& r) const
{
    const OperatorType& mat = *this->mat_;

    typedef typename OperatorType::row_type RowType;
    const RowType& row = mat[index];

    typedef typename RowType::ConstIterator ColumnIterator;

    r = (*this->rhs_)[index];

    /* The following loop subtracts
     * \f[ sum_i = \sum_j A_{ij}w_j \f]
     */
    ColumnIterator cIt    = row.begin();
    ColumnIterator cEndIt = row.end();
    
    for (; cIt!=cEndIt; ++cIt) {
        // r_i -= A_ij x_j
        cIt->mmv((*this->x_)[cIt.index()], r);
    }

}

template<class OperatorType, class DiscFuncType, class BitVectorType>
inline
void BlockGSStep<OperatorType, DiscFuncType, BitVectorType>::iterate()
{
    const OperatorType& mat = *this->mat_;

    assert(this->ignoreNodes_ != NULL);
    for (size_t i=0; i<this->x_->size(); i++) {
        size_t const count = (*this->ignoreNodes_)[i].count();
        if (count == BlockSize)
            continue;

        VectorBlock r;
        residual(i, r);

        // Compute x_i += A_{i,i}^{-1} r[i]
        VectorBlock v;
        VectorBlock& x = (*this->x_)[i];

        if (count == 0) {
            // No degree of freedom shall be ignored --> solve linear problem
            mat[i][i].solve(v, r);
        } else {
            // Copy the matrix and adjust rhs and matrix so that dofs given in ignoreNodes[i]
            // are not touched
            typename OperatorType::block_type matRes;
            for (int j = 0; j < BlockSize; ++j) {
                if ((*this->ignoreNodes_)[i][j]) {
                    r[j] = 0;

                    for (int k = 0; k < BlockSize; ++k)
			matRes[j][k] = (k == j);
                } else 
                    matRes[j] = mat[i][i][j];

            }
            matRes.solve(v, r);
        }
        // Add correction;
        x += v;
    }
}
