template <class MatrixType, class VectorType>
void CGStep<MatrixType, VectorType>::check() const
{
    if (preconditioner_)
        preconditioner_->check();
}

template <class MatrixType, class VectorType>
void CGStep<MatrixType, VectorType>::preprocess()
{
    // Compute the residual (r starts out as the rhs)
    matrix_.mmv(x_,r_);

    if (preconditioner_) {
        preconditioner_->setMatrix(matrix_);
        preconditioner_->apply(p_, r_);
    } else
        p_ = r_;

    r_squared_old_ = p_*r_;
}

template <class MatrixType, class VectorType>
void CGStep<MatrixType, VectorType>::iterate()
{
    VectorType q(x_);

    matrix_.mv(p_, q);                             // q_0     = Ap_0
    const double alpha = r_squared_old_ / (p_ * q); // alpha_0 = r_0*r_0/p_0*Ap_0
    x_.axpy(alpha, p_);                            // x_1     = x_0 + alpha_0 p_0
    r_.axpy(-alpha, q);                            // r_1     = r_0 - alpha_0 Ap_0

    if (preconditioner_)
        preconditioner_->apply(q, r_);
    else
        q = r_;

    const double r_squared = q * r_;
    const double beta = r_squared / r_squared_old_; // beta_0 = r_1*r_1/ (r_0*r_0)
    p_ *= beta;                                    // p_1 = r_1 + beta_0 p_0
    p_ += q;
    r_squared_old_ = r_squared;
}
