#ifndef DUNE_LINE_GAUSS_SEIDEL_STEP_HH
#define DUNE_LINE_GAUSS_SEIDEL_STEP_HH

#include <dune/common/bitsetvector.hh>

#include <dune/solvers/iterationsteps/blockgsstep.hh>

template<class OperatorType,
         class DiscFuncType,
         class BitVectorType = Dune::BitSetVector<DiscFuncType::block_type::dimension> >
         class LineGSStep : public BlockGSStep<OperatorType, DiscFuncType, BitVectorType>
    {

        typedef typename DiscFuncType::block_type VectorBlock;

        enum {BlockSize = VectorBlock::dimension};

    protected:

        // Describes the ordering of the degrees of freedom in 1d blocks
        std::vector<std::vector<unsigned int> > blockStructure_;

    public:
		
        //! Default constructor.  Doesn't init anything
        LineGSStep() {}

        //! Constructor for usage of Permutation Manager
        LineGSStep( const std::vector<std::vector<unsigned int> >& blockStructure)
            : blockStructure_( blockStructure )
        {}

        //! Constructor with a linear problem
        LineGSStep(const OperatorType& mat, DiscFuncType& x, DiscFuncType& rhs)
            : BlockGSStep<OperatorType,DiscFuncType>(mat, x, rhs)
        {}
		
        //! Perform one iteration
        virtual void iterate();

        virtual DiscFuncType getSol();

    };


#include "linegsstep.cc"

#endif
