#ifndef DUNE_MONOTONE_MULTIGRID_STEP_HH
#define DUNE_MONOTONE_MULTIGRID_STEP_HH

#include <dune/solvers/iterationsteps/multigridstep.hh>
#include <dune/solvers/iterationsteps/projectedblockgsstep.hh>
#include <dune/solvers/transferoperators/multigridtransfer.hh>

#include <dune/solvers/common/boxconstraint.hh>
#include <dune/solvers/transferoperators/obstaclerestrictor.hh>

/** \brief The general monotone multigrid solver
*/
template<class MatrixType, class VectorType>
class MonotoneMGStep : public MultigridStep<MatrixType, VectorType>
{
    
    static const int dim = VectorType::block_type::dimension;
    
    typedef typename VectorType::field_type field_type;
    
public:
    
    MonotoneMGStep() {}
    
    MonotoneMGStep(int numLevels)
    DUNE_DEPRECATED_MSG("The number of levels is no longer set explicitely, but instead inferred from the number of transfer operators. Just erase the number of levels from the argument list in your constructor call.")
    {}
    
    MonotoneMGStep(const MatrixType& mat, 
                   VectorType& x, 
                   VectorType& rhs, int numLevels)
    DUNE_DEPRECATED_MSG("The number of levels is no longer set explicitely, but instead inferred from the number of transfer operators. Just erase the number of levels from the argument list in your constructor call.")
        : MultigridStep<MatrixType, VectorType>(mat, x, rhs, numLevels)
    {
        oldCritical.resize(x.size(), false);
    }
    
    MonotoneMGStep(const MatrixType& mat,
                   VectorType& x,
                   VectorType& rhs)
        : MultigridStep<MatrixType, VectorType>(mat, x, rhs)
    {
        oldCritical.resize(x.size(), false);
    }

    virtual ~MonotoneMGStep() {}
    
    virtual void setProblem(const MatrixType& mat,
                            VectorType& x,
                            VectorType& rhs,
                            int numLevels)
    DUNE_DEPRECATED_MSG("Use setProblem(const MatrixType& mat,VectorType& x,const VectorType& rhs). The number of levels is no longer set explicitely, but instead inferred from the number of transfer operators.")
    {
        MultigridStep<MatrixType, VectorType>::setProblem(mat,x,rhs);
        oldCritical.resize(x.size(), false);
    }

    virtual void setProblem(const MatrixType& mat,
                            VectorType& x,
                            VectorType& rhs)
    {
        MultigridStep<MatrixType, VectorType>::setProblem(mat,x,rhs);
        oldCritical.resize(x.size(), false);
    }

    virtual void iterate();
    
    virtual void preprocess();
    
    virtual void nestedIteration();
    
    ObstacleRestrictor<VectorType>* obstacleRestrictor_;
    
    std::vector<Dune::BitSetVector<1> >* hasObstacle_;
    
    std::vector<Dune::BitSetVector<1> > recompute_;
    
    std::vector<std::vector<BoxConstraint<field_type,dim> > >* obstacles_;
    
    // Needed to track changes in the set of critical bits, and allows
    // to check which dofs where critical after the last iteration
    Dune::BitSetVector<dim> oldCritical;
};

#include "mmgstep.cc"

#endif
