#ifndef DUNE_PROJECTED_BLOCK_GAUSS_SEIDEL_STEP_HH
#define DUNE_PROJECTED_BLOCK_GAUSS_SEIDEL_STEP_HH

#include <vector>

#include "blockgsstep.hh"
#include <dune/solvers/common/boxconstraint.hh>

template<class OperatorType, class DiscFuncType>
class ProjectedBlockGSStep : public BlockGSStep<OperatorType, DiscFuncType>
{
    
    typedef typename DiscFuncType::block_type VectorBlock;
    
    typedef typename DiscFuncType::field_type field_type;

    enum {BlockSize = VectorBlock::dimension};
    
public:
    
    //! Default constructor.  Doesn't init anything
    ProjectedBlockGSStep() {}
    
    //! Constructor with a linear problem
    ProjectedBlockGSStep(const OperatorType& mat, DiscFuncType& x, const DiscFuncType& rhs)
        : BlockGSStep<OperatorType,DiscFuncType>(mat, x, rhs)
    {}

    //! Perform one iteration
    virtual void iterate();
    
    virtual DiscFuncType getSol();
    
    Dune::BitSetVector<1>* hasObstacle_;
    
    const std::vector<BoxConstraint<field_type,BlockSize> >* obstacles_;
};

#include "projectedblockgsstep.cc"

#endif
