#ifndef DUNE_TRUST_REGION_BLOCK_GAUSS_SEIDEL_STEP_HH
#define DUNE_TRUST_REGION_BLOCK_GAUSS_SEIDEL_STEP_HH

/** \file
    \brief A Gauss-Seidel iterator which does not assume that the functional
    is convex, but which expects a bounded admissible set.
*/

#include "projectedblockgsstep.hh"
#include <dune/solvers/common/boxconstraint.hh>

    /** \brief A Gauss-Seidel iterator which does not assume that the functional
        is convex, but which expects a bounded admissible set.
    */
    template<class MatrixType, class VectorType>
    class TrustRegionGSStep : public ProjectedBlockGSStep<MatrixType, VectorType>
    {

        typedef typename VectorType::block_type VectorBlock;

        enum {blocksize = VectorBlock::dimension};

        typedef typename VectorType::field_type field_type;

    public:

        //! Default constructor.  Doesn't init anything
        TrustRegionGSStep() {}

        //! Constructor with a linear problem
        TrustRegionGSStep(MatrixType& mat, VectorType& x, VectorType& rhs)
            : LinearIterationStep<MatrixType,VectorType>(mat, x, rhs)
        {}

        //! Perform one iteration
        virtual void iterate();

        virtual VectorType getSol();

        field_type residual(unsigned int index, int blockIndex) const;

    };

#include "trustregiongsstep.cc"

#endif
